import java.util.*;

public class Esame{
  
  
  static public class Squadra implements Comparable{
    protected String nome; 
    protected int nPunti;
    protected int diffReti;
    
    Squadra(String nome, int nPunti, int diffReti){
      this.nome = nome;
      this.nPunti = nPunti;
      this.diffReti = diffReti;
    }
    
    String getNome(){
      return nome;
    }
    
    int getPunti(){
      return nPunti;
    }
    
    int getDiffReti(){
      return diffReti;
    }
    
    
    public boolean equals(Object obj) {
	    if (obj instanceof Squadra) {
		    Squadra s = (Squadra) obj;
		    return ((nome.equals(s.nome)) && (nPunti == s.nPunti) && (diffReti == s.diffReti)); 
	    }
	    return false;
    }

        
/**
    * Ridefinisce il metodo compareTo considerando: il campo nome, il campo nPunti ed il campo diffReti (punti 25)
    * Il metodo al momento ritorna -1. Modificare il metodo in modo da ritornare 
    * un intero negativo/positivo se l'oggetto di invocazione e' minore/maggiore del parametro, oppure zero se i due oggetti sono uguali.
    * Una squadra a e' minore/maggiore di una squadra b se il campo nPunti e' minore del campo nPunti di b.
    * Se il campo nPunti di a e' uguale al campo nPunti di b allora a e' minore di b se il campo diffReti di a e' minore del campo diffReti di b
    * Se sia il campo nPunti che il campo diffReti di a e di b sono uguali allora una squadra a e' minore di una squadra b se il campo nome di a e' minore del campo nome di b (secondo l'ordine lessicografico). Il metodo deve lanciare un'eccezione di tipo IllegalArgumentException se il parametro non e' di tipo Squadra e quindi non puo' essere comparato con l'oggetto di invocazione
    *  
    *@param obj oggetto con cui comparare l'oggetto di invocazione
    *@return un intero negativo/positivo o zero se l'oggetto di invocazione e' minore/maggiore o uguale al parametro
    *@exception lancia una eccezione di tipo IllegalArgumentException se il parametro non e' di tipo Squadra e quindi non puo' essere comparato con l'oggetto di invocazione 
    */
    public int compareTo(Object obj) throws IllegalArgumentException {
	    if (obj instanceof Squadra) {
		    Squadra s = (Squadra) obj;
		    if (nPunti==s.nPunti){
			if (diffReti==s.diffReti){
			    return nome.compareTo(s.nome);
			} else {
			    return diffReti-s.diffReti;
			}
		    } else { 
			return nPunti-s.nPunti;
		    }
	    }
	    throw new IllegalArgumentException("l'oggetto non e' di tipo Squadra");
	
    }
        
    public int hashCode(){
      return nome.hashCode()+nPunti+diffReti;
    }

    public String toString(){
      String res = "< ";
      res+=nome;
      res+=", "+nPunti;
      res+=", "+diffReti;
      res+= " >";
      return res;
    }

  
  }
  
  
  /**
  * Dato una insieme di Squadre, ritornare la squadra con il numero massimo di diffReti e che abbia totalizzato almeno n punti(punti 30).
  * Se ci sono piu' squadre con la stessa diff reti, ritornare una qualsiasi delle squadre
  * Se l'insieme e' vuoto o se non ci sono squadre che hanno totalizzato almeno n punti restituire null
  * Es: dato S = <<Brasile,7,5><Messico,7,3><Croazia,3,0>,<Cameroon,0,-8>> ed n = 7 il risultato sara' <Brasile,7,5> 
  * Es: dato S = <<Brasile,7,5><Messico,7,3><Croazia,3,0>,<Cameroon,0,-8>> ed n = 10 il risultato sara' null
  *
  * @param S insieme di squadre
  * @return n numero minimo di punti
  */
  public static Squadra squadraMigliore(Set<Squadra> S,int n){
   Squadra best = null;	
    for(Iterator<Squadra> its = S.iterator();its.hasNext();){
	  Squadra s = its.next();
	  if ( s.getPunti() >= n  && ( best==null || s.getDiffReti()>best.getDiffReti() ) ){
	     best = s;
     	  }
    }
    return best;
  } 
  
  
  /**
  * Dato un grafo g, un nodo di partenza n, ed un insieme di nodi S, restituisce true se tutti i nodi contenuti in S sono raggiungibili da n (punti 20)
  * 
  * @param g grafo
  * @param n nodo del grafo
  * @param S insieme di nodi
  * @return un insieme di nodi che hanno distanza minima pari a d da n
  */
  public static  boolean raggiungibili(Grafo g, Object n, Set<Object> S) {
    Set<Object> ragg = new HashSet<Object>();
    dfs(g,n,ragg);
    return ragg.containsAll(S);
  }

  private static void dfs(Grafo g,Object n,Set<Object> r){
     r.add(n);
     Set as = g.getEdgeSet(n);
     for(Iterator it = as.iterator(); it.hasNext();){
	Arco a = (Arco) it.next();
	if (!r.contains(a.getNode1())){
	    dfs(g,a.getNode1(),r);
	} 
	if (!r.contains(a.getNode2())){
	    dfs(g,a.getNode2(),r);      
	}
     }
  }


  
  /**
  * Dato un grafo g, ed un intero n restituire l'insieme dei nodi con grado maggiore o uguale ad n (punti 25)
  * Il grado di un nodo e' il numero dei suoi vicini. 
  * Se il grafo non  ha nessun nodo restituisce l'insieme vuoto. 
  * Si possono usare i metodi getNodes() e getDegree(Object node) della classe Grafo.java per ottenere rispettivamente l'insieme dei nodi  
  * del grafo ed il grado di un nodo. 
  *
  * @param g grafo
  * @return nodo di massimo grado o null se grafo senza nodi.
  */    
  public static Set<Object> nodiGradoMaggiore(Grafo g, int n){
     Set<Object> nodes = g.getNodeSet(); 
     Set<Object> ret = new HashSet<Object>(); 
     for (Iterator<Object> nit = nodes.iterator(); nit.hasNext();){
	Object node = nit.next();
	if (g.getDegree(node)>= n){
	  ret.add(node);
	}
     } 
     return ret;
  }

  public static void main(String[] args){
    
    
    System.out.println("**********TEST compareTo*************");
    Squadra s1 = new Squadra("Brasile",7,5);
    Squadra s2 = new Squadra("Messico",7,3);
    Squadra s3 = new Squadra("Croazia",3,0);
    Squadra s4 = new Squadra("Cameroon",0,-8);


    Squadra s5 = new Squadra("Messico",7,3);


    System.out.println("s1 "+s1+" compareTo s2 "+s2+" = "+ s1.compareTo(s2)+"[dovrebbe essere positivo]");	
    
    System.out.println("s2 "+s2+" compareTo s5 "+s5+" = "+ s2.compareTo(s5)+"[dovrebbe essere 0]");	

    System.out.println("s4 "+s4+" compareTo s3 "+s3+" = "+ s4.compareTo(s3)+"[dovrebbe essere negativo]");	

    try {
	System.out.println("s1 "+s1+" compareTo 3 = "+ s1.compareTo(new Integer(3)));
    } catch (IllegalArgumentException e) {
	System.out.println(e.getMessage());	
    }		


    System.out.println("**********TEST squadraMigliore*************");


   Set<Squadra> squadre = new HashSet<Squadra>();
   squadre.add(s1); squadre.add(s2); squadre.add(s3); squadre.add(s4);    

   System.out.println("insieme in input " + squadre + " punti = 7");
    
   System.out.println("squadra migliore =  " + squadraMigliore(squadre,7)+" [dovrebbe essere <Brasile,7,5>]");


   System.out.println("insieme in input " + squadre + " punti = 10");
    
   System.out.println("squadra migliore =  " + squadraMigliore(squadre,10) +" [dovrebbe essere null]");



   Set<Squadra> vuoto = new HashSet<Squadra>();
   System.out.println("insieme in input " + vuoto  + " punti = 0 ");
    
   System.out.println("squadra migliore = " + squadraMigliore(vuoto,0)+" [dovrebbe essere null]");

   System.out.println("**********TEST raggiungibili*************");

	    Object na = "a"; Object nb = "b"; Object nc = "c"; Object nd = "d"; Object ne = "e"; Object nf = "f"; Object nh = "h"; Object nk = "k";

	    Grafo g = new Grafo();    
	    g.add(na,nb,new Integer(1));
	    g.add(nb,nc,new Integer(1));
	    g.add(nb,nd,new Integer(1));
	    g.add(nc,nd,new Integer(1));
	    g.add(nd,ne,new Integer(1));
	    g.add(nd,nf,new Integer(1));
	    g.add(nh);	
    	    g.add(nk);	
	    g.add(nh,nk,new Integer(1));
    
	    Set<Object> nodi = new HashSet<Object>(); 
	    nodi.add(na); nodi.add(nb); nodi.add(nc); nodi.add(nd); nodi.add(ne); nodi.add(nf); 

	    boolean testRagg1 = raggiungibili(g,na,nodi);
	    

	    boolean testRagg2 = raggiungibili(g,nh,nodi);

	    System.out.println("Il grafo G e':\n" + g);
	    System.out.println("Gli elementi "+nodi+ "sono raggiungibili da "+na+" ? "+testRagg1+"[dovrebbe essere true]");
	    System.out.println("Gli elementi "+nodi+ "sono raggiungibili da "+nh+" ? "+testRagg2+"[dovrebbe essere false]");



    System.out.println("**********TEST maxdegree*************");
 
    System.out.println("nodi con grado maggiore o uguale a 2: "+nodiGradoMaggiore(g,2)+" [dovrebbero essere b,c,d]");
  
    System.out.println("nodi con grado maggiore o uguale a 1: "+nodiGradoMaggiore(g,1)+" [dovrebbero essere a,b,c,d,e,f,h,k]");

    System.out.println("nodi con grado maggiore o uguale a 5: "+nodiGradoMaggiore(g,5)+" [dovrebbe essere vuoto]");

  }

}
